<?php

// TODO Set our own cache bin.
define('VIEWS_NAVIGATION_CACHE_BIN', 'cache');

/**
 * Get a view query from cache.
 */
function views_navigation_get_cached_query($cid) {
  $cache = cache_get('query-' . $cid, VIEWS_NAVIGATION_CACHE_BIN);
  // ATM we only handle the default views query backend.
  if ($cache && is_a($cache->data, 'views_plugin_query_default')) {
    return $cache->data;
  }
}

/**
 * Get a view result from cache.
 */
function views_navigation_get_cached_result($cid) {
  $cache = cache_get('result-' . $cid, VIEWS_NAVIGATION_CACHE_BIN);
  return $cache ? $cache->data : FALSE;
}

/**
 * Store a view query in cache.
 * Return the cid or FALSE if the query is not stored.
 */
function views_navigation_store_query($view) {
  // ATM we only handle the default views query backend.
  if (!is_a($view->query, 'views_plugin_query_default')) {
    return FALSE;
  }

  $query_to_store = clone($view->query);
  $view_to_store = clone($query_to_store->view);

  // Handle the case when there is no pager.
  if (!isset($view_to_store->total_rows)) {
    $view_to_store->total_rows = count($view_to_store->result);
  }

  // If there is zero or one result, we do nothing.
  if ($view_to_store->total_rows < 2) {
    return FALSE;
  }

  // Store the cycling option value.
  $view_to_store->cycle = $view->display_handler->get_option('views_navigation_cycle');

  // Remove useless properties. This is the not easy part.
  unset($query_to_store->display);
  unset($query_to_store->header);
  unset($query_to_store->limit);
  unset($query_to_store->pager);
  foreach ($view_to_store as $key => $value) {
    if (!in_array($key, array(
      'base_database',
      'name',
      'total_rows',
      'cycle',
    ))) {
      unset($view_to_store->$key);
    }
  }

  $query_to_store->view = $view_to_store;

  $cid = views_navigation_get_query_cid($query_to_store);
  // The query may be stored already.
  if (!views_navigation_get_cached_query($cid)) {
    cache_set('query-' . $cid, $query_to_store, VIEWS_NAVIGATION_CACHE_BIN);
  }
  return $cid;
}

/**
 * Get the unique cid corresponding to a view query.
 */
function views_navigation_get_query_cid($query) {
  return drupal_hash_base64(serialize($query));
}

/**
 * Redirect to the next or previous node.
 */
function views_navigation_router($cid, $pos) {
  if ($result = views_navigation_get_result($cid)) {
    // Manage array ends, cycling behavior.
    $max_index = count($result) - 1;
    $pos = ($pos > $max_index) ? 0 : (($pos < 0) ? $max_index : $pos);

    if ($nid = $result[$pos]) {
      $params = array(
        'vnc' => $cid,
        'vnp' => $pos,
      );
      drupal_goto('node/' . $nid, array('query' => $params));
    }
  }
  // Something went wrong.
  watchdog('views_navigation', t('Could not find any next/previous item.'), func_get_args(), WATCHDOG_ERROR);
  return t("Sorry, could not find any next/previous item.");
}

/**
 * Get the result of a query, as an array of nids keyed by position.
 */
function views_navigation_get_result($cid) {
  if (!$result = views_navigation_get_cached_result($cid)) {
    if ($query = views_navigation_get_cached_query($cid)) {
      $real_query = $query->query();
      $rows = $real_query->execute();
      $result = array();
      foreach ($rows as $pos => $row) {
        $result[$pos] = $row->nid;
      }
    }
  }
  return $result;
}

/**
 * Build and render the previous/next links for the node being viewed.
 */
function views_navigation_get_links($view_mode = 'full') {

  // Ensure we can show links.
  $cid = isset($_GET['vnc']) ? $_GET['vnc'] : NULL;
  $pos = isset($_GET['vnp']) ? $_GET['vnp'] : NULL;
  if (!isset($cid) || !isset($pos) || $view_mode != 'full') {
    return FALSE;
  }
  $query = views_navigation_get_cached_query($cid);
  if (!$query) {
    return FALSE;
  }

  if ($query->view->cycle || $pos > 0) {
    $links['previous'] = array(
      'title' => t('Previous'),
      'href' => 'views_navigation/' . $cid . '/' . ($pos - 1),
    );
  }

  if ($query->view->cycle || $pos < $query->view->total_rows - 1) {
    $links['next'] = array(
      'title' => t('Next'),
      'href' => 'views_navigation/' . $cid . '/' . ($pos + 1),
    );
  }

  return array(
    '#theme' => 'links',
    '#links' => $links,
  );
}

/**
 * Add the query parameters to append to the node url.
 * The view must have been executed already.
 */
function _views_navigation_build_query($nid, $view, $query = array()) {
  if (isset($view->views_navigation_cid)) {
    //TODO handle the case when the nid is in the result more than one
    // time.
    $nids = array();
    foreach ($view->result as $result) {
      $nids[] .= $result->nid;
    }
    $index = array_search($nid, $nids);
    //Take care of pagination. Index of first item on second page must be
    // items_per_page, not 0.
    $pager = $view->query->pager;
    if ($pager->use_pager()) {
      $index += ($pager->get_items_per_page() * $pager->get_current_page());
    }

    $query = array_merge($query, array(
      'vnc' => $view->views_navigation_cid,
      'vnp' => $index,
    ));
  }
  return $query;
}

/**
 * Used when the view handler needs an already built url. This is the case when
 * it will be passed to parse_url() as in views_handler_field::render_as_link().
 */
function _views_navigation_build_url($nid, $view, $options = array('absolute' => TRUE)) {
  $options['query'] = _views_navigation_build_query($nid, $view, isset($options['query']) ? $options['query'] : array());
  return url("node/$nid", $options);
}
